/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.bindings;

import static org.objectweb.fractal.adl.types.TypeInterfaceUtil.isClient;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.components.Component;
import org.objectweb.fractal.adl.interfaces.InterfaceContainer;
import org.objectweb.fractal.adl.types.TypeInterface;

/**
 * Exception thrown when an interface that must be bound, is not.
 */
public class InterfaceNotBoundException extends ADLException {

  private final TypeInterface      itf;
  private final InterfaceContainer itfContainer;

  /**
   * Construct a new {@link InterfaceNotBoundException}.
   * 
   * @param itf the interface that should be bound.
   * @param itfContainer the interface container that contains the not bound
   *            interface.
   */
  public InterfaceNotBoundException(final TypeInterface itf,
      final InterfaceContainer itfContainer) {
    super("Interface not bound", itf);
    this.itf = itf;
    this.itfContainer = itfContainer;
  }

  @Override
  public String getMessage() {
    final String compName;
    if (itfContainer instanceof Component)
      compName = ((Component) itfContainer).getName();
    else
      compName = ((Definition) itfContainer).getName();

    if (isClient(itf)) {
      return "The client interface \"" + itf.getName()
          + "\" of the component \"" + compName
          + "\" is not bound. (interface defined at: "
          + ((Node) itf).astGetSource() + "; component defined at: "
          + ((Node) itfContainer).astGetSource() + ")";
    } else {
      return "The server interface \"" + itf.getName()
          + "\" of the composite \"" + compName
          + "\" is not bound inside the composite. This interface must be "
          + "bound to a sub component of the composite, "
          + "or must be declared optional. (interface defined at: "
          + ((Node) itf).astGetSource() + "; component defined at: "
          + ((Node) itfContainer).astGetSource() + ")";
    }
  }
}
