/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;

import org.objectweb.fractal.adl.util.Printer;

public final class ExecutionHelper {
  private ExecutionHelper() {
  }

  public static int exec(final String command) throws Exception {
    return exec(command.split("\\s+"));
  }

  public static int exec(final String... args) throws Exception {
    final Process process = Runtime.getRuntime().exec(args);
    return waitFor(process);
  }

  public static int exec(final ProcessBuilder processBuilder) throws Exception {
    return waitFor(processBuilder.start());
  }

  private static int waitFor(final Process process) throws InterruptedException {
    final InputStream perr = process.getErrorStream();

    // pipe error stream of process in Printer;
    new Thread(new Runnable() {
      public void run() {
        final LineNumberReader reader = new LineNumberReader(
            new InputStreamReader(new BufferedInputStream(perr)));
        String line;
        try {
          while ((line = reader.readLine()) != null) {
            Printer.error(line);
          }
          reader.close();
        } catch (final IOException e) {
          throw new InternalError(e.getMessage());
        }
      }
    }).start();

    return process.waitFor();
  }
}
