/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.controllers;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.implementations.Controller;
import org.objectweb.fractal.adl.implementations.ControllerContainer;
import org.objectweb.fractal.adl.implementations.Implementation;
import org.objectweb.fractal.adl.implementations.ImplementationContainer;
import org.objectweb.fractal.cecilia.adl.components.PrimitiveChecker;
import org.objectweb.fractal.cecilia.adl.implementations.ExtendedImplementation;
import org.objectweb.fractal.cecilia.adl.plugin.AbstractPluginUser;
import org.objectweb.fractal.cecilia.adl.plugin.PluginException;

/**
 * Add the controllers description in the AST.
 */
public class ControllerCheckerSelector extends AbstractPluginUser
    implements
      PrimitiveChecker {

  // --------------------------------------------------------------------------
  // Client interfaces
  // --------------------------------------------------------------------------

  /** The name of the {@link PrimitiveChecker} client collection interface. */
  public static final String CONTROLLER_CHECKERS = "controller-checker";

  /** Default constructor. */
  public ControllerCheckerSelector() {
    super(CONTROLLER_CHECKERS);
  }

  // --------------------------------------------------------------------------
  // Implementation of the PrimitiveChecker interface
  // --------------------------------------------------------------------------

  public void check(final List<ComponentContainer> path,
      final ComponentContainer container, final Map<Object, Object> context)
      throws ADLException {
    boolean composite = false;

    if (!(container instanceof ImplementationContainer))
      throw new ADLException(
          "Component AST node is not instance of "
              + "\""
              + org.objectweb.fractal.adl.implementations.ImplementationContainer.class
                  .getCanonicalName() + "\". " + "Check the DTD", container);
    final Implementation implementation = ((ImplementationContainer) container)
        .getImplementation();
    if (implementation == null)
      composite = true;
    else if (!(implementation instanceof ExtendedImplementation))
      throw new ADLException(
          "Invalid controller node in AST: Must be an instance of "
              + "\""
              + org.objectweb.fractal.cecilia.adl.implementations.ExtendedImplementation.class
                  .getCanonicalName() + "\"" + ". Check the DTD", container);
    final ExtendedImplementation extImplementation = (ExtendedImplementation) implementation;

    if (!(container instanceof ControllerContainer))
      throw new ADLException("Component AST node is not instance of " + "\""
          + org.objectweb.fractal.adl.components.ComponentContainer.class
          + "\". " + "Check the DTD", container);
    final Controller controller = ((ControllerContainer) container)
        .getController();
    if (controller == null)
      throw new ADLException("Can't find controller node in AST", container);
    if (!(controller instanceof ExtendedController))
      throw new ADLException(
          "Invalid controller node in AST: Must be an instance of "
              + "'"
              + org.objectweb.fractal.cecilia.adl.controllers.ExtendedController.class
                  .getCanonicalName() + "'" + ". Check the DTD", container);
    final ExtendedController extController = (ExtendedController) controller;

    final StringBuffer sb = new StringBuffer();
    if (composite)
      sb.append("composite.");
    else
      sb.append("primitive.").append(extImplementation.getLanguage()).append(
          '.');

    sb.append(extController.getLanguage().substring(0, 1).toUpperCase())
        .append(extController.getLanguage().substring(1)).append(
            extController.getDescriptor().substring(0, 1).toUpperCase())
        .append(extController.getDescriptor().substring(1));

    PrimitiveChecker controllerChecker;
    try {
      controllerChecker = getPlugin(sb.toString(), context,
          PrimitiveChecker.class);
    } catch (final PluginException e) {
      throw new ADLException(
          "Can't find controller checker plugin for description \'"
              + controller.getDescriptor() + "\'", controller, e);
    }
    controllerChecker.check(path, container, context);
  }
}
