/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan, Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.implementations;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.ComponentVisitor;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.implementations.ImplementationContainer;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.cecilia.adl.components.AbstractComponentVisitorPluginUser;
import org.objectweb.fractal.cecilia.adl.plugin.PluginException;
import org.objectweb.fractal.task.core.TaskException;

/**
 * Delegates compilation to a client {@link ComponentVisitor} depending on the
 * implementation language of the component for which its visit method is
 * invoked.
 */
public class VisitorSelectorForLanguage
    extends
      AbstractComponentVisitorPluginUser implements ComponentVisitor {

  // --------------------------------------------------------------------------
  // Implementation of the ComponentVisitor interface
  // --------------------------------------------------------------------------

  public Component visit(final List<Node> path, final ComponentContainer node,
      final Map<Object, Object> context) throws ADLException, TaskException {
    String implementationLanguage = null;
    if (node instanceof ImplementationContainer) {
      ExtendedImplementation implem = null;
      if (((ImplementationContainer) node).getImplementation() instanceof ExtendedImplementation)
        implem = (ExtendedImplementation) ((ImplementationContainer) node)
            .getImplementation();
      if (implem != null) {
        implementationLanguage = implem.getLanguage();
      }

      if (implementationLanguage == null)
        throw new ADLException(
            "Can't find the implementation language of the component", implem,
            null);
    } else
      throw new ADLException("Can't find the implementation of the component",
          node, null);

    try {
      return getVisitorPlugin(implementationLanguage, context).visit(path,
          node, context);
    } catch (final PluginException e) {
      throw new ADLException("Can't find plugin for implementation language '"
          + implementationLanguage + "'", node, e);
    }
  }
}
